#pragma once

#include "scalar.h"
#include "vector.h"

/* Matrix: a 3x3 matrix */
class Matrix {
public:
    //Scalar data[9];     ...or...
    //Scalar data[3][3];  ...or...
    Vector x, y, z;   // the three columns


    Matrix(Vector c0, Vector c1, Vector c2) : x(c0), y(c1), z(c2) {}

    // method to access to the column as if Matrix was an array of 3 Vectors
    Vector operator[](int index) const {
        const Vector* v = (const Vector*)this;
        return v[index];
    }

    // same, but for left values (values "on the left" of an assignment, values being assigned)
    Vector& operator[](int index) {
        Vector* v = (Vector*)this;
        return v[index];
    }


    Vector col(int index) const { return (*this)[index]; }
    Vector row(int index) const { return Vector(x[index], y[index], z[index]); }

    void transpose() {
        std::swap(x.y, y.x);
        std::swap(z.x, x.z);
        std::swap(y.z, z.y);
    }

    Matrix transposed() const {
        return Matrix(row(0), row(1), row(2));
    }

    void invert();  // In-place TODO

    Matrix inverse() const;  // Out-of-place  TODO


};

inline bool isEqual(Matrix m1, Matrix m2) {
    return isEqual(m1.x, m2.x) && isEqual(m1.y, m2.y) && isEqual(m1.z, m2.z);
}
